@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -55,7 +45,7 @@ get_tunits(const char *unit, int &incperiod, int &incunit, int &tunit)
   else if (memcmp(unit, "days", len) == 0)    { incunit = 86400; tunit = TUNIT_DAY; }
   else if (memcmp(unit, "months", len) == 0)  { incunit = 1;     tunit = TUNIT_MONTH; }
   else if (memcmp(unit, "years", len) == 0)   { incunit = 12;    tunit = TUNIT_YEAR; }
-  else cdoAbort("Time unit >%s< unsupported!", unit);
+  else cdo_abort("Time unit >%s< unsupported!", unit);
 
   if (tunit == TUNIT_HOUR)
     {
@@ -77,18 +67,18 @@ shifttime(int calendar, int tunit, int64_t ijulinc, int64_t &vdate, int &vtime)
       cdiDecodeDate(vdate, &year, &month, &day);
 
       month += (int) ijulinc;
-      adjustMonthAndYear(month, year);
+      adjust_month_and_year(month, year);
 
       vdate = cdiEncodeDate(year, month, day);
     }
   else
     {
-      auto juldate = julianDateEncode(calendar, vdate, vtime);
-      juldate = julianDateAddSeconds(ijulinc, juldate);
-      julianDateDecode(calendar, juldate, vdate, vtime);
+      auto juldate = juldate_encode(calendar, vdate, vtime);
+      juldate = juldate_add_seconds(ijulinc, juldate);
+      juldate_decode(calendar, juldate, vdate, vtime);
 
       if (Options::cdoVerbose)
-        cdoPrint("juldate, ijulinc, vdate, vtime: %g %lld %lld %d", julianDateToSeconds(juldate), ijulinc, vdate, vtime);
+        cdo_print("juldate, ijulinc, vdate, vtime: %g %lld %lld %d", juldate_to_seconds(juldate), ijulinc, vdate, vtime);
     }
 }
 
@@ -122,14 +112,14 @@ time_gen_bounds(int calendar, int tunit, int incperiod, int64_t vdate, int vtime
   else if (tunit == TUNIT_DAY)
     {
       vdateb[0] = vdate;
-      auto juldate = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
-      juldate = julianDateAddSeconds(86400, juldate);
-      julianDateDecode(calendar, juldate, vdateb[1], vtimeb[1]);
+      auto juldate = juldate_encode(calendar, vdateb[0], vtimeb[0]);
+      juldate = juldate_add_seconds(86400, juldate);
+      juldate_decode(calendar, juldate, vdateb[1], vtimeb[1]);
     }
   else if (tunit == TUNIT_HOUR || tunit == TUNIT_3HOURS || tunit == TUNIT_6HOURS || tunit == TUNIT_12HOURS)
     {
       if (incperiod == 0) incperiod = 1;
-      if (incperiod > 24) cdoAbort("Time period must be less equal 24!");
+      if (incperiod > 24) cdo_abort("Time period must be less equal 24!");
 
       // clang-format off
       if      (tunit == TUNIT_3HOURS)  incperiod = 3;
@@ -145,9 +135,9 @@ time_gen_bounds(int calendar, int tunit, int incperiod, int64_t vdate, int vtime
       if (h1 >= 24)
         {
           vdateb[1] = cdiEncodeDate(year, month, day + 1);
-          auto juldate = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
-          juldate = julianDateAddSeconds(incperiod * 3600, juldate);
-          julianDateDecode(calendar, juldate, vdateb[1], vtimeb[1]);
+          auto juldate = juldate_encode(calendar, vdateb[0], vtimeb[0]);
+          juldate = juldate_add_seconds(incperiod * 3600, juldate);
+          juldate_decode(calendar, juldate, vdateb[1], vtimeb[1]);
         }
       else
         vtimeb[1] = cdiEncodeTime(h1, 0, 0);
@@ -158,7 +148,7 @@ int
 evaluateCalendarString(int operatorID, const std::string &calendarName)
 {
   int calendar = CALENDAR_STANDARD;
-  const auto calendarString = stringToLower(calendarName);
+  const auto calendarString = string_to_lower(calendarName);
   // clang-format off
   if      (calendarString == "standard")  calendar = CALENDAR_STANDARD;
   else if (calendarString == "gregorian") calendar = CALENDAR_GREGORIAN;
@@ -170,7 +160,7 @@ evaluateCalendarString(int operatorID, const std::string &calendarName)
   else if (calendarString == "365_day") calendar = CALENDAR_365DAYS;
   else if (calendarString == "366days") calendar = CALENDAR_366DAYS;
   else if (calendarString == "366_day") calendar = CALENDAR_366DAYS;
-  else cdoAbort("Calendar >%s< unsupported! Available %s", calendarName.c_str(), cdoOperatorEnter(operatorID));
+  else cdo_abort("Calendar >%s< unsupported! Available %s", calendarName.c_str(), cdo_operator_enter(operatorID));
   // clang-format on
 
   return calendar;
@@ -187,7 +177,7 @@ decode_datestr(const char *datestr)
     }
   else
     {
-      return parameter2long(datestr);
+      return parameter_to_long(datestr);
     }
 }
 
@@ -202,7 +192,7 @@ decode_timestr(const char *timestr)
     }
   else
     {
-      return parameter2int(timestr);
+      return parameter_to_int(timestr);
     }
 }
 
@@ -219,23 +209,21 @@ decode_timeunits(const char *timeunits, int &incperiod, int &incunit, int &tunit
   get_tunits(timeunits, incperiod, incunit, tunit);
 }
 
-
 static void
 argument2datetimeinc(int64_t &sdate, int &stime, int &incperiod, int &incunit, int &tunit)
 {
-  if (operatorArgc() < 1) cdoAbort("Too few arguments!");
-  if (operatorArgc() > 3) cdoAbort("Too many arguments!");
+  if (cdo_operator_argc() < 1) cdo_abort("Too few arguments!");
+  if (cdo_operator_argc() > 3) cdo_abort("Too many arguments!");
 
-  sdate = decode_datestr(cdoOperatorArgv(0).c_str());
+  sdate = decode_datestr(cdo_operator_argv(0).c_str());
 
-  if (operatorArgc() > 1)
+  if (cdo_operator_argc() > 1)
     {
-      stime = decode_timestr(cdoOperatorArgv(1).c_str());
-      if (operatorArgc() == 3) decode_timeunits(cdoOperatorArgv(2).c_str(), incperiod, incunit, tunit);
+      stime = decode_timestr(cdo_operator_argv(1).c_str());
+      if (cdo_operator_argc() == 3) decode_timeunits(cdo_operator_argv(2).c_str(), incperiod, incunit, tunit);
     }
 }
 
-
 void *
 Settime(void *process)
 {
@@ -255,26 +243,26 @@ Settime(void *process)
   // int nargs;
   JulianDate juldate;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto SETYEAR     = cdoOperatorAdd("setyear",      0,  1, "year");
-  const auto SETMON      = cdoOperatorAdd("setmon",       0,  1, "month");
-  const auto SETDAY      = cdoOperatorAdd("setday",       0,  1, "day");
-  const auto SETDATE     = cdoOperatorAdd("setdate",      0,  1, "date (format: YYYY-MM-DD)");
-  const auto SETTIME     = cdoOperatorAdd("settime",      0,  1, "time (format: hh:mm:ss)");
-  const auto SETTUNITS   = cdoOperatorAdd("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
-  const auto SETTAXIS    = cdoOperatorAdd("settaxis",     0, -2, "date<,time<,increment>> (format YYYY-MM-DD,hh:mm:ss)");
-  const auto SETTBOUNDS  = cdoOperatorAdd("settbounds",   0,  1, "frequency (day, month, year)");
-  const auto SETREFTIME  = cdoOperatorAdd("setreftime",   0, -2, "date<,time<,units>> (format YYYY-MM-DD,hh:mm:ss)");
-  const auto SETCALENDAR = cdoOperatorAdd("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
-  const auto SHIFTTIME   = cdoOperatorAdd("shifttime",    0,  1, "shift value");
+  const auto SETYEAR     = cdo_operator_add("setyear",      0,  1, "year");
+  const auto SETMON      = cdo_operator_add("setmon",       0,  1, "month");
+  const auto SETDAY      = cdo_operator_add("setday",       0,  1, "day");
+  const auto SETDATE     = cdo_operator_add("setdate",      0,  1, "date (format: YYYY-MM-DD)");
+  const auto SETTIME     = cdo_operator_add("settime",      0,  1, "time (format: hh:mm:ss)");
+  const auto SETTUNITS   = cdo_operator_add("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
+  const auto SETTAXIS    = cdo_operator_add("settaxis",     0, -2, "date<,time<,increment>> (format YYYY-MM-DD,hh:mm:ss)");
+  const auto SETTBOUNDS  = cdo_operator_add("settbounds",   0,  1, "frequency (day, month, year)");
+  const auto SETREFTIME  = cdo_operator_add("setreftime",   0, -2, "date<,time<,units>> (format YYYY-MM-DD,hh:mm:ss)");
+  const auto SETCALENDAR = cdo_operator_add("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
+  const auto SHIFTTIME   = cdo_operator_add("shifttime",    0,  1, "shift value");
   // clang-format on
 
-  const auto operatorID = cdoOperatorID();
-  // nargs = cdoOperatorF2(operatorID);
+  const auto operatorID = cdo_operator_id();
+  // nargs = cdo_operator_f2(operatorID);
 
-  operatorInputArg(cdoOperatorEnter(operatorID));
+  operator_input_arg(cdo_operator_enter(operatorID));
 
   if (operatorID == SETTAXIS || operatorID == SETREFTIME)
     {
@@ -284,52 +272,52 @@ Settime(void *process)
     }
   else if (operatorID == SETDATE)
     {
-      operatorCheckArgc(1);
-      const auto datestr = cdoOperatorArgv(0).c_str();
+      operator_check_argc(1);
+      const auto datestr = cdo_operator_argv(0).c_str();
       newval = decode_datestr(datestr);
     }
   else if (operatorID == SETTIME)
     {
-      operatorCheckArgc(1);
-      const auto timestr = cdoOperatorArgv(0).c_str();
+      operator_check_argc(1);
+      const auto timestr = cdo_operator_argv(0).c_str();
       newval = decode_timestr(timestr);
     }
   else if (operatorID == SHIFTTIME)
     {
-      operatorCheckArgc(1);
-      decode_timeunits(cdoOperatorArgv(0).c_str(), incperiod, incunit, tunit);
+      operator_check_argc(1);
+      decode_timeunits(cdo_operator_argv(0).c_str(), incperiod, incunit, tunit);
       // increment in seconds
       ijulinc = (int64_t) incperiod * incunit;
     }
   else if (operatorID == SETTUNITS || operatorID == SETTBOUNDS)
     {
-      operatorCheckArgc(1);
-      decode_timeunits(cdoOperatorArgv(0).c_str(), incperiod, incunit, tunit);
+      operator_check_argc(1);
+      decode_timeunits(cdo_operator_argv(0).c_str(), incperiod, incunit, tunit);
 
       if (operatorID == SETTBOUNDS
           && !(tunit == TUNIT_HOUR || tunit == TUNIT_3HOURS || tunit == TUNIT_6HOURS || tunit == TUNIT_12HOURS || tunit == TUNIT_DAY
                || tunit == TUNIT_MONTH || tunit == TUNIT_YEAR))
-        cdoAbort("Unsupported frequency %s! Use hour, 3hours, 6hours, day, month or year.", cdoOperatorArgv(0).c_str());
+        cdo_abort("Unsupported frequency %s! Use hour, 3hours, 6hours, day, month or year.", cdo_operator_argv(0).c_str());
     }
   else if (operatorID == SETCALENDAR)
     {
-      operatorCheckArgc(1);
-      auto cname = cdoOperatorArgv(0);
+      operator_check_argc(1);
+      auto cname = cdo_operator_argv(0);
       newcalendar = evaluateCalendarString(operatorID, cname);
     }
   else
     {
-      operatorCheckArgc(1);
-      newval = parameter2int(cdoOperatorArgv(0));
+      operator_check_argc(1);
+      newval = parameter_to_int(cdo_operator_argv(0));
     }
 
-  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
   const auto vlistID2 = vlistDuplicate(vlistID1);
 
   const auto taxisID1 = vlistInqTaxis(vlistID1);
-  bool taxis_has_bounds = taxisHasBounds(taxisID1) > 0;
+  auto taxis_has_bounds = (taxisHasBounds(taxisID1) > 0);
   auto ntsteps = vlistNtsteps(vlistID1);
   const auto nvars = vlistNvars(vlistID1);
 
@@ -349,7 +337,7 @@ Settime(void *process)
 
   const auto calendar = taxisInqCalendar(taxisID1);
 
-  if (Options::cdoVerbose) cdoPrint("calendar = %d", calendar);
+  if (Options::cdoVerbose) cdo_print("calendar = %d", calendar);
 
   if (operatorID == SETREFTIME)
     {
@@ -357,13 +345,13 @@ Settime(void *process)
 
       if (taxisInqType(taxisID1) == TAXIS_ABSOLUTE)
         {
-          cdoPrint("Changing absolute to relative time axis!");
-          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
+          cdo_print("Changing absolute to relative time axis!");
+          taxisID2 = cdo_taxis_create(TAXIS_RELATIVE);
         }
       else
         taxisID2 = taxisDuplicate(taxisID1);
 
-      if (operatorArgc() != 3) tunit = taxisInqTunit(taxisID1);
+      if (cdo_operator_argc() != 3) tunit = taxisInqTunit(taxisID1);
       taxisDefTunit(taxisID2, tunit);
     }
   else if (operatorID == SETTUNITS)
@@ -372,8 +360,8 @@ Settime(void *process)
 
       if (taxisInqType(taxisID1) == TAXIS_ABSOLUTE)
         {
-          cdoPrint("Changing absolute to relative time axis!");
-          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
+          cdo_print("Changing absolute to relative time axis!");
+          taxisID2 = cdo_taxis_create(TAXIS_RELATIVE);
           taxisDefTunit(taxisID2, tunit);
         }
       else
@@ -382,12 +370,12 @@ Settime(void *process)
   else if (operatorID == SETCALENDAR)
     {
       copy_timestep = true;
-      //if ( ((char *)argument)[0] == '-' ) cdoAbort("This operator does not work with pipes!");
+      // if ( ((char *)argument)[0] == '-' ) cdo_abort("This operator does not work with pipes!");
       if (taxisInqType(taxisID1) == TAXIS_ABSOLUTE)
         {
-          // if ( CdoDefault::FileType != CDI_FILETYPE_NC ) cdoAbort("This operator does not work on an absolute time axis!");
-          cdoPrint("Changing absolute to relative time axis!");
-          taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
+          // if ( CdoDefault::FileType != CDI_FILETYPE_NC ) cdo_abort("This operator does not work on an absolute time axis!");
+          cdo_print("Changing absolute to relative time axis!");
+          taxisID2 = cdo_taxis_create(TAXIS_RELATIVE);
         }
       else
         taxisID2 = taxisDuplicate(taxisID1);
@@ -400,7 +388,7 @@ Settime(void *process)
       taxisDefTunit(taxisID2, tunit);
       taxisDefRdate(taxisID2, sdate);
       taxisDefRtime(taxisID2, stime);
-      juldate = julianDateEncode(calendar, sdate, stime);
+      juldate = juldate_encode(calendar, sdate, stime);
     }
   else if (operatorID == SETTUNITS)
     {
@@ -418,7 +406,7 @@ Settime(void *process)
   if (operatorID != SHIFTTIME)
     if (taxis_has_bounds && !copy_timestep)
       {
-        cdoWarning("Time bounds unsupported by this operator, removed!");
+        cdo_warning("Time bounds unsupported by this operator, removed!");
         taxisDeleteBounds(taxisID2);
         taxis_has_bounds = false;
       }
@@ -427,14 +415,15 @@ Settime(void *process)
 
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
-  auto gridsizemax = vlistGridsizeMax(vlistID1);
-  if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-  Varray<double> array(gridsizemax);
+  VarList varList1;
+  varListInit(varList1, vlistID1);
+
+  Field field;
 
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       auto vdate = taxisInqVdate(taxisID1);
@@ -453,17 +442,15 @@ Settime(void *process)
               else
                 {
                   month += (int) ijulinc;
-                  adjustMonthAndYear(month, year);
-
+                  adjust_month_and_year(month, year);
                   day = (day0 == 31) ? days_per_month(calendar, year, month) : day0;
-
                   vdate = cdiEncodeDate(year, month, day);
                 }
             }
           else
             {
-              julianDateDecode(calendar, juldate, vdate, vtime);
-              juldate = julianDateAddSeconds(ijulinc, juldate);
+              juldate_decode(calendar, juldate, vdate, vtime);
+              juldate = juldate_add_seconds(ijulinc, juldate);
             }
         }
       else if (operatorID == SETTBOUNDS)
@@ -472,11 +459,11 @@ Settime(void *process)
 
           if (Options::CMOR_Mode)
             {
-              const auto juldate1 = julianDateEncode(calendar, vdateb[0], vtimeb[0]);
-              const auto juldate2 = julianDateEncode(calendar, vdateb[1], vtimeb[1]);
-              const auto seconds = julianDateToSeconds(julianDateSub(juldate2, juldate1)) / 2;
-              const auto juldatem = julianDateAddSeconds(lround(seconds), juldate1);
-              julianDateDecode(calendar, juldatem, vdate, vtime);
+              const auto juldate1 = juldate_encode(calendar, vdateb[0], vtimeb[0]);
+              const auto juldate2 = juldate_encode(calendar, vdateb[1], vtimeb[1]);
+              const auto seconds = juldate_to_seconds(juldate_sub(juldate2, juldate1)) / 2;
+              const auto juldatem = juldate_add_seconds(lround(seconds), juldate1);
+              juldate_decode(calendar, juldatem, vdate, vtime);
             }
         }
       else if (operatorID == SHIFTTIME)
@@ -492,7 +479,6 @@ Settime(void *process)
         }
       else if (operatorID == SETREFTIME || operatorID == SETCALENDAR || operatorID == SETTUNITS)
         {
-          ;
         }
       else
         {
@@ -500,9 +486,9 @@ Settime(void *process)
 
           if (operatorID == SETYEAR) year = newval;
           if (operatorID == SETMON) month = newval;
-          if (operatorID == SETMON && (month < 0 || month > 16)) cdoAbort("parameter month=%d out of range!", month);
+          if (operatorID == SETMON && (month < 0 || month > 16)) cdo_abort("parameter month=%d out of range!", month);
           if (operatorID == SETDAY) day = newval;
-          if (operatorID == SETDAY && (day < 0 || day > 31)) cdoAbort("parameter day=%d out of range!", day);
+          if (operatorID == SETDAY && (day < 0 || day > 31)) cdo_abort("parameter day=%d out of range!", day);
 
           vdate = cdiEncodeDate(year, month, day);
 
@@ -512,7 +498,7 @@ Settime(void *process)
 
       if (copy_timestep)
         {
-          taxisCopyTimestep(taxisID2, taxisID1);
+          cdo_taxis_copy_timestep(taxisID2, taxisID1);
           if (operatorID == SETREFTIME)
             {
               taxisDefRdate(taxisID2, sdate);
@@ -536,30 +522,30 @@ Settime(void *process)
 
       if (streamID2 == CDO_STREAM_UNDEF)
         {
-          streamID2 = cdoOpenWrite(1);
-          cdoDefVlist(streamID2, vlistID2);
+          streamID2 = cdo_open_write(1);
+          cdo_def_vlist(streamID2, vlistID2);
         }
 
-      cdoDefTimestep(streamID2, tsID);
+      cdo_def_timestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
-          cdoInqRecord(streamID1, &varID, &levelID);
-          cdoDefRecord(streamID2, varID, levelID);
+          cdo_inq_record(streamID1, &varID, &levelID);
+          cdo_def_record(streamID2, varID, levelID);
 
-          size_t nmiss;
-          cdoReadRecord(streamID1, &array[0], &nmiss);
-          cdoWriteRecord(streamID2, &array[0], nmiss);
+          field.init(varList1[varID]);
+          cdo_read_record(streamID1, field);
+          cdo_write_record(streamID2, field);
         }
 
       tsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
