@@ -88,6 +88,9 @@ void   *Copy(void *argument);
 void   *DCW_util(void *argument);
 #define DCW_utilOperators                 {"dcw"}
 
+void   *Dayarith(void *argument);
+#define DayarithOperators                 {"dayadd", "daysub", "daymul", "daydiv"}
+
 void   *Deltat(void *argument);
 #define DeltatOperators                   {"deltat", "timederivative"}
 
