@@ -41,6 +41,7 @@ static const module_t module_Condc          = {"Condc"          ,Condc
 static const module_t module_Consecstat     = {"Consecstat"     ,Consecstat     , ConsecstatHelp    , ConsecstatOperators     , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Copy           = {"Copy"           ,Copy           , CopyHelp          , CopyOperators           , EXPOSED  , CDI_REAL , -1 , 1     , NoRestriction };
 static const module_t module_DCW_util       = {"DCW_util"       ,DCW_util       , {}                , DCW_utilOperators       , EXPOSED  , CDI_REAL , 0  , 0     , NoRestriction };
+static const module_t module_Dayarith       = {"Dayarith"       ,Dayarith       , DayarithHelp      , DayarithOperators       , EXPOSED  , CDI_REAL , 2  , 1     , NoRestriction };
 static const module_t module_Deltat         = {"Deltat"         ,Deltat         , DeltatHelp        , DeltatOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Deltime        = {"Deltime"        ,Deltime        , {}                , DeltimeOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Depth          = {"Depth"          ,Depth          , {}                , DepthOperators          , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
@@ -155,8 +156,8 @@ static const module_t module_Selyearidx     = {"Selyearidx"     ,Selyearidx
 static const module_t module_Set            = {"Set"            ,Set            , SetHelp           , SetOperators            , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction ,  {Alias ("setvar", "setname")}};
 static const module_t module_Setattribute   = {"Setattribute"   ,Setattribute   , SetattributeHelp  , SetattributeOperators   , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Setbox         = {"Setbox"         ,Setbox         , SetboxHelp        , SetboxOperators         , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
-static const module_t module_Setgatt        = {"Setgatt"        ,Setgatt        , {}                , SetgattOperators        , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
 static const module_t module_Setgrid        = {"Setgrid"        ,Setgrid        , SetgridHelp       , SetgridOperators        , EXPOSED  , CDI_BOTH , 1  , 1     , NoRestriction };
+static const module_t module_Setgridcell    = {"Setgridcell"    ,Setgridcell    , SetgridcellHelp   , SetgridcellOperators    , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Sethalo        = {"Sethalo"        ,Sethalo        , SethaloHelp       , SethaloOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Setmiss        = {"Setmiss"        ,Setmiss        , SetmissHelp       , SetmissOperators        , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
 static const module_t module_Setmisstonn    = {"Setmisstonn"    ,Fillmiss       , SetmissHelp       , SetmisstonnOperators    , EXPOSED  , CDI_REAL , 1  , 1     , NoRestriction };
