@@ -1,17 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 #ifdef _OPENMP
@@ -105,12 +96,12 @@ nlat_to_nlon_cubic(int nlat)
 }
 
 static void
-scale_vec(double scalefactor, size_t nvals, double *values)
+scale_vec(double scalefactor, size_t n, double *values)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nvals, scalefactor, values)
+#pragma omp parallel for if (n > 99999) default(shared) schedule(static)
 #endif
-  for (size_t n = 0; n < nvals; ++n) values[n] *= scalefactor;
+  for (size_t i = 0; i < n; ++i) values[i] *= scalefactor;
 }
 
 static void
@@ -158,7 +149,7 @@ grid_to_radian(const char *units, size_t nvals, double *values, const char *desc
     }
   else
     {
-      cdoWarning("Unknown units [%s] supplied for %s; proceeding assuming radians!", units, description);
+      cdo_warning("Unknown units [%s] supplied for %s; proceeding assuming radians!", units, description);
     }
 }
 
@@ -167,7 +158,7 @@ grid_to_degree(const char *units, size_t nvals, double *values, const char *desc
 {
   if (strncmp(units, "rad", 3) == 0)
     {
-      for (size_t n = 0; n < nvals; ++n) values[n] *= RAD2DEG;
+      scale_vec(RAD2DEG, nvals, values);
     }
   else if (strncmp(units, "deg", 3) == 0)
     {
@@ -175,7 +166,7 @@ grid_to_degree(const char *units, size_t nvals, double *values, const char *desc
     }
   else
     {
-      cdoWarning("Unknown units [%s] supplied for %s; proceeding assuming degress!", units, description);
+      cdo_warning("Unknown units [%s] supplied for %s; proceeding assuming degress!", units, description);
     }
 }
 
@@ -227,7 +218,7 @@ gridToZonal(const int gridID1)
     }
   else
     {
-      cdoAbort("Gridtype %s unsupported!", gridNamePtr(gridtype));
+      cdo_abort("Gridtype %s unsupported!", gridNamePtr(gridtype));
     }
 
   return gridID2;
@@ -257,7 +248,7 @@ gridToMeridional(const int gridID1)
     }
   else
     {
-      cdoAbort("Gridtype %s unsupported!", gridNamePtr(gridtype));
+      cdo_abort("Gridtype %s unsupported!", gridNamePtr(gridtype));
     }
 
   return gridID2;
@@ -405,7 +396,7 @@ void
 grid_gen_xbounds2D(size_t nx, size_t ny, const std::vector<double> &xbounds, std::vector<double> &xbounds2D)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nx, ny, xbounds, xbounds2D)
+#pragma omp parallel for default(none) shared(nx, ny, xbounds, xbounds2D) schedule(static)
 #endif
   for (size_t i = 0; i < nx; ++i)
     {
@@ -427,7 +418,7 @@ void
 grid_gen_ybounds2D(size_t nx, size_t ny, const std::vector<double> &ybounds, std::vector<double> &ybounds2D)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nx, ny, ybounds, ybounds2D)
+#pragma omp parallel for default(none) shared(nx, ny, ybounds, ybounds2D) schedule(static)
 #endif
   for (size_t j = 0; j < ny; ++j)
     {
@@ -546,7 +537,7 @@ qu2reg_subarea(size_t gridsize, int np, double xfirst, double xlast, double *arr
   int wlen;
   int ii;
 
-  if (np <= 0) cdoAbort("Number of values between pole and equator missing!");
+  if (np <= 0) cdo_abort("Number of values between pole and equator missing!");
 
   grib_get_reduced_row(np4, xfirst, xlast, &row_count, &ilon_firstx, &ilon_last);
   int nx = row_count;
@@ -593,7 +584,7 @@ qu2reg_subarea(size_t gridsize, int np, double xfirst, double xlast, double *arr
       size += row_count;
     }
 
-  if (gridsize != size) cdoAbort("gridsize1 inconsistent! (gridsize=%zu found=%zu)", gridsize, size);
+  if (gridsize != size) cdo_abort("gridsize1 inconsistent! (gridsize=%zu found=%zu)", gridsize, size);
 
   qu2reg3_double(work, reducedPoints, ny, np4, missval, iret, lmiss, lperio, lveggy);
 
@@ -637,11 +628,20 @@ get_xfirst_and_xlast(int gridID, double &xfirst, double &xlast)
     }
 }
 
+static bool
+reduced_grid_is_global(int np, int nxmax, double xfirst, double xlast)
+{
+  const auto dx_global = (np > 0) ? (90.0 / np) : 999.0;
+  auto dx_data = 360.0 - (xlast - xfirst);
+  if ((dx_data > dx_global) && (dx_data * nxmax > 360.0)) dx_data = 360.0 / nxmax;
+  return !(dx_data > dx_global);
+}
+
 void
 field2regular(int gridID1, int gridID2, double missval, double *array, size_t nmiss, int lnearest)
 {
   const auto gridtype = gridInqType(gridID1);
-  if (gridtype != GRID_GAUSSIAN_REDUCED) cdoAbort("Not a reduced Gaussian grid!");
+  if (gridtype != GRID_GAUSSIAN_REDUCED) cdo_abort("Not a reduced Gaussian grid!");
 
   auto nx = gridInqXsize(gridID1);
   const auto ny = gridInqYsize(gridID1);
@@ -653,31 +653,32 @@ field2regular(int gridID1, int gridID2, double missval, double *array, size_t nm
   double xfirst = 0.0, xlast = 359.9999;
   if (nx == 2) get_xfirst_and_xlast(gridID1, xfirst, xlast);
 
+  int nxmax = 0;
+  for (size_t i = 0; i < ny; ++i) nxmax = std::max(nxmax, reducedPoints[i]);
+
   const int lmiss = (nmiss > 0);
   const int lperio = 1;
 
   int iret;
-  const auto dx_global = (np > 0) ? (90.0 / np) : 999.0;
-  const auto dx_data = 360.0 - (xlast - xfirst);
-  if (dx_data > dx_global)
-    {
-      nx = qu2reg_subarea(gridInqSize(gridID1), np, xfirst, xlast, array, reducedPoints.data(), ny, missval, &iret, lmiss, lperio,
-                          lnearest);
-    }
-  else
+  if (reduced_grid_is_global(np, nxmax, xfirst, xlast))
     {
       nx = gridInqXsize(gridID2);
       qu2reg3_double(array, reducedPoints.data(), ny, nx, missval, &iret, lmiss, lperio, lnearest);
     }
+  else
+    {
+      nx = qu2reg_subarea(gridInqSize(gridID1), np, xfirst, xlast, array, reducedPoints.data(), ny, missval, &iret, lmiss, lperio,
+                          lnearest);
+    }
 
-  if (gridInqSize(gridID2) != nx * ny) cdoAbort("Gridsize differ!");
+  if (gridInqSize(gridID2) != nx * ny) cdo_abort("Gridsize differ!");
 }
 
 int
 gridToRegular(int gridID1)
 {
   const auto gridtype = gridInqType(gridID1);
-  if (gridtype != GRID_GAUSSIAN_REDUCED) cdoAbort("Not a reduced Gaussian grid!");
+  if (gridtype != GRID_GAUSSIAN_REDUCED) cdo_abort("Not a reduced Gaussian grid!");
 
   auto nx = gridInqXsize(gridID1);
   const auto ny = gridInqYsize(gridID1);
@@ -686,14 +687,25 @@ gridToRegular(int gridID1)
   std::vector<double> xvals, yvals(ny);
   gridInqYvals(gridID1, yvals.data());
 
+  std::vector<int> reducedPoints(ny);
+  gridInqReducedPoints(gridID1, reducedPoints.data());
+
   double xfirst = 0.0, xlast = 359.9999;
   if (nx == 2) get_xfirst_and_xlast(gridID1, xfirst, xlast);
 
-  const auto dx_global = (np > 0) ? (90.0 / np) : 999.0;
-  const auto dx_data = 360.0 - (xlast - xfirst);
-  if (dx_data > dx_global)
+  int nxmax = 0;
+  for (size_t i = 0; i < ny; ++i) nxmax = std::max(nxmax, reducedPoints[i]);
+
+  if (reduced_grid_is_global(np, nxmax, xfirst, xlast))
     {
-      if (np <= 0) cdoAbort("Number of values between pole and equator missing!");
+      nx = reducedPoints[ny / 2];
+      if (nx < (2 * ny)) cdo_abort("Number of longitudes %zu is less than 2*ny=%zu!", nx, ny * 2);
+      xvals.resize(nx);
+      for (size_t i = 0; i < nx; ++i) xvals[i] = xfirst + i * 360.0 / nx;
+    }
+  else
+    {
+      if (np <= 0) cdo_abort("Number of values between pole and equator missing!");
 
       // sub area (longitudes)
       const auto np4 = np * 4;
@@ -708,15 +720,6 @@ gridToRegular(int gridID1)
           if (xfirst > xlast) xvals[i] -= 360.0;
         }
     }
-  else
-    {
-      std::vector<int> reducedPoints(ny);
-      gridInqReducedPoints(gridID1, reducedPoints.data());
-      nx = reducedPoints[ny / 2];
-      if (nx < (2 * ny)) cdoAbort("Number of longitudes %zu is less than 2*ny=%zu!", nx, ny * 2);
-      xvals.resize(nx);
-      for (size_t i = 0; i < nx; ++i) xvals[i] = xfirst + i * 360.0 / nx;
-    }
 
   const auto gridsize = nx * ny;
   const auto gridID2 = gridCreate(GRID_GAUSSIAN, gridsize);
@@ -835,12 +838,12 @@ static void
 check_units(const char *name, const char *units)
 {
   const auto len = strlen(units);
-  const bool lvalid_units
+  const auto unitsIsValid
       = (len == 1 && *units == 'm') || (len == 2 && memcmp(units, "km", 2) == 0) || (strncmp(units, "meter", 5) == 0);
 
-  if (!lvalid_units)
-    cdoWarning("Possibly wrong result! %s %s-coordinate units: %s%s%s (expected \"m\" or \"km\"; default \"m\")",
-               len ? "Invalid" : "Missing", name, len ? "\"" : "", units, len ? "\"" : "");
+  if (!unitsIsValid)
+    cdo_warning("Possibly wrong result! %s %s-coordinate units: %s%s%s (expected \"m\" or \"km\"; default \"m\")",
+                len ? "Invalid" : "Missing", name, len ? "\"" : "", units, len ? "\"" : "");
 }
 
 static void
@@ -902,7 +905,7 @@ get_projection(int gridID1)
       char gmapname[CDI_MAX_NAME];
       int length = CDI_MAX_NAME;
       cdiInqKeyString(gridID1, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, gmapname, &length);
-      cdoAbort("Projection type >%s< unsupported!", gmapname);
+      cdo_abort("Projection type >%s< unsupported!", gmapname);
     }
   // clang-format on
 
@@ -910,7 +913,7 @@ get_projection(int gridID1)
 }
 
 static void
-applyProjection(Projection projection, std::vector<char> &proj_params, int gridID, size_t n, double *x, double *y)
+apply_projection(Projection projection, std::vector<char> &proj_params, int gridID, size_t n, double *x, double *y)
 {
   // clang-format off
   if      (projection == Projection::proj_sinu)   cdo_sinu_to_lonlat(n, x, y);
@@ -926,17 +929,17 @@ gridToCurvilinear(int gridID1, int lbounds)
 {
   const auto gridtype = gridInqType(gridID1);
   if (!(gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_PROJECTION))
-    cdoAbort("%s: Grid type >%s< unsupported!", __func__, gridNamePtr(gridtype));
+    cdo_abort("%s: Grid type >%s< unsupported!", __func__, gridNamePtr(gridtype));
 
   auto nx = gridInqXsize(gridID1);
   auto ny = gridInqYsize(gridID1);
 
   const bool lxyvals = gridHasCoordinates(gridID1);
-  if (!lxyvals) cdoAbort("Grid coordinates missing!");
+  if (!lxyvals) cdo_abort("Grid coordinates missing!");
 
   const auto gridsize = gridInqSize(gridID1);
   const auto gridID2 = gridCreate(GRID_CURVILINEAR, gridsize);
-  gridDefDatatype(gridID2, CDI_DATATYPE_FLT32);
+  cdiDefKeyInt(gridID2, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT32);
 
   Projection projection(Projection::none);
   std::vector<char> proj_params;
@@ -1002,7 +1005,7 @@ gridToCurvilinear(int gridID1, int lbounds)
       if (projection != Projection::none)
         {
           gridDefProj(gridID2, gridID1);
-          applyProjection(projection, proj_params, gridID1, gridsize, xvals2D.data(), yvals2D.data());
+          apply_projection(projection, proj_params, gridID1, gridsize, xvals2D.data(), yvals2D.data());
         }
     }
 
@@ -1019,7 +1022,7 @@ gridToCurvilinear(int gridID1, int lbounds)
           gridInqXbounds(gridID1, xbounds.data());
           if (check_range(2 * nx, xbounds.data(), -720, 720))
             {
-              cdoWarning("longitude bounds out of range, skipped!");
+              cdo_warning("longitude bounds out of range, skipped!");
               xbounds.clear();
             }
         }
@@ -1035,7 +1038,7 @@ gridToCurvilinear(int gridID1, int lbounds)
           gridInqYbounds(gridID1, ybounds.data());
           if (check_range(2 * ny, ybounds.data(), -180, 180))
             {
-              cdoWarning("latitude bounds out of range, skipped!");
+              cdo_warning("latitude bounds out of range, skipped!");
               ybounds.clear();
             }
         }
@@ -1065,7 +1068,7 @@ gridToCurvilinear(int gridID1, int lbounds)
                 {
                   bounds_1D_to_2D(nx, ny, xbounds, ybounds, xbounds2D, ybounds2D, xscale, yscale);
                   if (projection != Projection::none)
-                    applyProjection(projection, proj_params, gridID1, 4 * gridsize, xbounds2D.data(), ybounds2D.data());
+                    apply_projection(projection, proj_params, gridID1, 4 * gridsize, xbounds2D.data(), ybounds2D.data());
                 }
               else
                 {
@@ -1260,7 +1263,7 @@ gridToUnstructuredGaussianReduced(int gridID1, int gridID2, size_t gridsize, int
     }
   else
     {
-      cdoAbort("%s: latitude coordinates missing!", gridNamePtr(gridInqType(gridID1)));
+      cdo_abort("%s: latitude coordinates missing!", gridNamePtr(gridInqType(gridID1)));
     }
 
   std::vector<double> xvals(gridsize);
@@ -1377,7 +1380,7 @@ gridToUnstructured(int gridID1, int lbounds)
   auto gridtype = gridInqType(gridID1);
   const auto gridsize = gridInqSize(gridID1);
 
-  bool lproj_rll = false;
+  auto lproj_rll = false;
   if (gridtype == GRID_PROJECTION && gridInqProjType(gridID1) == CDI_PROJ_RLL)
     {
       gridtype = GRID_LONLAT;
@@ -1415,12 +1418,12 @@ gridToUnstructured(int gridID1, int lbounds)
       }
     default:
       {
-        cdoAbort("Grid type >%s< unsupported!", gridNamePtr(gridtype));
+        cdo_abort("Grid type >%s< unsupported!", gridNamePtr(gridtype));
         break;
       }
     }
 
-  gridDefDatatype(gridID2, CDI_DATATYPE_FLT32);
+  cdiDefKeyInt(gridID2, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT32);
 
   return gridID2;
 }
@@ -1429,7 +1432,7 @@ int
 gridCurvilinearToRegular(int gridID1)
 {
   int gridID2 = -1;
-  bool lx = true, ly = true;
+  auto lx = true, ly = true;
 
   const auto gridtype = gridInqType(gridID1);
   const auto gridsize = gridInqSize(gridID1);
@@ -1476,7 +1479,7 @@ gridCurvilinearToRegular(int gridID1)
       gridDefXsize(gridID2, nx);
       gridDefYsize(gridID2, ny);
 
-      //  gridDefDatatype(gridID2, CDI_DATATYPE_FLT32);
+      // cdiDefKeyInt(gridID2, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT32);
 
       cdo_grid_to_degree(gridID2, CDI_XAXIS, nx, xvals.data(), "grid1 center lon");
       cdo_grid_to_degree(gridID2, CDI_YAXIS, ny, yvals.data(), "grid1 center lat");
@@ -1504,7 +1507,7 @@ gridGenWeights(int gridID, double *grid_area, double *grid_wgts)
       gridInqMaskGME(gridID, grid_mask.data());
     }
 
-  double total_area = 0;
+  double total_area = 0.0;
   int nvals = 0;
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -1514,7 +1517,7 @@ gridGenWeights(int gridID, double *grid_area, double *grid_wgts)
       nvals++;
     }
 
-  if (gridVerbose) cdoPrint("Total area = %g", total_area);
+  if (gridVerbose) cdo_print("Total area = %g", total_area);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -1534,15 +1537,15 @@ gridGenWeights(int gridID, double *grid_area, double *grid_wgts)
 int
 gridWeights(const int gridID, double *grid_wgts)
 {
-  int w_status = 1;
-  int a_status = 0;
+  auto w_status = 1;
+  auto a_status = 0;
 
   const auto gridsize = gridInqSize(gridID);
   std::vector<double> grid_area(gridsize);
 
   if (gridHasArea(gridID))
     {
-      if (gridVerbose) cdoPrint("Using existing grid cell area!");
+      if (gridVerbose) cdo_print("Using existing grid cell area!");
       gridInqArea(gridID, grid_area.data());
     }
   else
@@ -1581,7 +1584,7 @@ gridWeights(const int gridID, double *grid_wgts)
 bool
 grid_is_distance_generic(int gridID)
 {
-  bool status = false;
+  auto status = false;
 
   if (gridInqType(gridID) == GRID_GENERIC)
     {
@@ -1604,11 +1607,11 @@ is_point_grid(int gridID)
   const auto projtype = gridInqProjType(gridID);
 
   const auto lprojection = (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL)
-                        || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC)
-                        || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_STERE);
+                           || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_LCC)
+                           || (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_STERE);
 
-  return (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR || lprojection ||
-          gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GME);
+  return (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR || lprojection
+          || gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GME);
 }
 
 static int
@@ -1632,7 +1635,7 @@ generate_full_grid(int gridID, bool withBounds)
     {
       const auto reference = dereferenceGrid(gridID);
       if (reference.isValid) gridID = reference.gridID;
-      if (reference.notFound) cdoAbort("Reference to source grid not found!");
+      if (reference.notFound) cdo_abort("Reference to source grid not found!");
     }
 
   return gridID;
@@ -1641,14 +1644,13 @@ generate_full_grid(int gridID, bool withBounds)
 int
 generate_full_point_grid(int gridID)
 {
-  constexpr bool withBounds = false;
+  constexpr auto withBounds = false;
   return generate_full_grid(gridID, withBounds);
 }
 
 int
 generate_full_cell_grid(int gridID)
 {
-  constexpr bool withBounds = true;
+  constexpr auto withBounds = true;
   return generate_full_grid(gridID, withBounds);
 }
-
